package main

import (
	"encoding/json"
	"net/http"

	"github.com/gin-gonic/gin"
	"github.com/gorilla/websocket"
)

// 连接成功结构体
type connectedDataS struct {
	Event            string `json:"_event"`
	HistoryClientNum int    `json:"historyClientNum"`
	OnlineClientNum  int    `json:"onlineClientNum"`
}

// 消息结构体
type messageDataS struct {
	Event   string `json:"_event"`
	Message string `json:"message"`
}

// ws upGrader
var upGrader = websocket.Upgrader{
	CheckOrigin: func(r *http.Request) bool {
		return true
	},
}

// 全部客户端
var clients = make(map[*websocket.Conn]bool)

// 存储数据
var (
	historyClientNum = 0
)

// 广播消息
func broadcastMsg(mt int, message []byte) {
	for client := range clients {
		client.WriteMessage(mt, message)
	}
}

// 存储当前客户端
func saveClient(client *websocket.Conn) {
	clients[client] = true
	// 更新历史客户端数
	historyClientNum++
	// 广播进入
	var data = connectedDataS{
		Event:            "connected",
		HistoryClientNum: historyClientNum,
		OnlineClientNum:  len(clients),
	}
	var dataJson, _ = json.Marshal(data)
	broadcastMsg(websocket.TextMessage, dataJson)
}

// 删除当前客户端
func delClient(client *websocket.Conn) {
	// 断开连接删除client
	delete(clients, client)
	// 广播离开
	var data = connectedDataS{
		Event:            "connected",
		HistoryClientNum: historyClientNum,
		OnlineClientNum:  len(clients),
	}
	var dataJson, _ = json.Marshal(data)
	broadcastMsg(websocket.TextMessage, dataJson)
}

// 发送消息
func sendMsg(mt int, message []byte) {
	var data = messageDataS{
		Event:   "message",
		Message: string(message),
	}
	dataJson, _ := json.Marshal(data)
	broadcastMsg(mt, dataJson)
}

// 创建客户端实例
func createWs(c *gin.Context) {
	// 创建ws连接客户端
	client, err := upGrader.Upgrade(c.Writer, c.Request, nil)
	if err != nil {
		return
	}
	// 连接成功后存储当前客户端
	saveClient(client)
	defer func() {
		client.Close()
		delClient(client)
	}()
	for {
		// 监听接受信息
		mt, message, err := client.ReadMessage()
		if err == nil {
			sendMsg(mt, message)
		} else {
			break
		}
	}
}

func main() {
	router := gin.Default()
	router.LoadHTMLFiles("index.html")
	router.GET("/", func(c *gin.Context) {
		c.HTML(http.StatusOK, "index.html", nil)
	})
	router.GET("/ws", createWs)
	router.Run(":9999")
}
